package ${basePackage}.module.system.mgr;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.support.CronSequenceGenerator;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ${basePackage}.frame.auth.Token;
import ${basePackage}.frame.base.ErrorType;
import ${basePackage}.frame.schedule.RunCronTask;
import ${basePackage}.frame.schedule.RunDelayRepeatTask;
import ${basePackage}.frame.schedule.RunFixRepeatTask;
import ${basePackage}.frame.schedule.Scheduler;
import ${basePackage}.frame.utils.IDgenerator;
import ${basePackage}.frame.utils.MapperUtil;
import ${basePackage}.frame.utils.Message;
import ${basePackage}.frame.utils.SqlUtil;
import ${basePackage}.frame.utils.ValidationUtil;
import ${basePackage}.module.system.ent.TaskSql;
import ${basePackage}.module.system.mpr.TaskSqlMapper;
import ${basePackage}.module.system.req.TaskSqlCreateRequest;
import ${basePackage}.module.system.req.TaskSqlDeleteRequest;
import ${basePackage}.module.system.req.TaskSqlFindRequest;
import ${basePackage}.module.system.req.TaskSqlGetRequest;
import ${basePackage}.module.system.req.TaskSqlUpdateRequest;
import ${basePackage}.module.system.rsp.TaskSqlCreateResponse;
import ${basePackage}.module.system.rsp.TaskSqlDeleteResponse;
import ${basePackage}.module.system.rsp.TaskSqlFindResponse;
import ${basePackage}.module.system.rsp.TaskSqlGetResponse;
import ${basePackage}.module.system.rsp.TaskSqlUpdateResponse;

import java.time.Duration;

/**
 * TASK_SQL - SQL任务
 *
 * @author author
 * @version 0.0.1
 * @since 2020-05-24
 */
@Transactional
@Service
public class TaskSqlManagerImpl implements TaskSqlManager {

    @Autowired
    private TaskSqlMapper taskSqlMapper;
    @Autowired
    private Scheduler scheduler;

    /**
     * 插入
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public TaskSqlCreateResponse create(TaskSqlCreateRequest request, Token token) {
        TaskSqlCreateResponse response = new TaskSqlCreateResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        if ("Cron".equals(request.getTaskType())) {
            try {
                new CronSequenceGenerator(request.getTypeValue());
            } catch (Exception e) {
                response.addError(ErrorType.BUSINESS_ERROR, "Cron表达式错误!");
                return response;
            }
        } else {
            String typeValue = request.getTypeValue();
            try {
                Integer.parseInt(typeValue);
            } catch (Exception e) {
                response.addError(ErrorType.BUSINESS_ERROR, "循环周期值错误!");
                return response;
            }
        }

        long id = IDgenerator.nextId();
        TaskSql entity = MapperUtil.map(request, TaskSql.class);
        entity.setId(id);

        long result = taskSqlMapper.insert(entity, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.CREATE_FAILURE);
            return response;
        }
        response.setId(id);
        createOrRepeat(entity);

        return response;
    }

    /**
     * 逻辑删除
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public TaskSqlDeleteResponse delete(TaskSqlDeleteRequest request, Token token) {
        TaskSqlDeleteResponse response = new TaskSqlDeleteResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long result = taskSqlMapper.delete(request, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.DELETE_FAILURE);
            return response;
        }
        response.setResult(result);
        scheduler.remove(String.valueOf(request.getId()));

        return response;
    }

    /**
     * 更新
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public TaskSqlUpdateResponse update(TaskSqlUpdateRequest request, Token token) {
        TaskSqlUpdateResponse response = new TaskSqlUpdateResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        if ("Cron".equals(request.getTaskType())) {
            try {
                new CronSequenceGenerator(request.getTypeValue());
            } catch (Exception e) {
                response.addError(ErrorType.BUSINESS_ERROR, "Cron表达式错误!");
                return response;
            }
        } else {
            String typeValue = request.getTypeValue();
            try {
                Integer.parseInt(typeValue);
            } catch (Exception e) {
                response.addError(ErrorType.BUSINESS_ERROR, "循环周期值错误!");
                return response;
            }
        }

        long result = taskSqlMapper.update(request, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.UPDATE_FAILURE);
            return response;
        }
        response.setResult(result);
        if (request.getValid()) {
            createOrRepeat(MapperUtil.map(request, TaskSql.class));
        } else {
            scheduler.remove(String.valueOf(request.getId()));
        }

        return response;
    }

    /**
     * 查询
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    @Transactional(readOnly = true)
    public TaskSqlFindResponse find(TaskSqlFindRequest request, Token token) {
        TaskSqlFindResponse response = new TaskSqlFindResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        if (request.getPageSize() != 0) {
            PageHelper.startPage(request.getPageNumber(), request.getPageSize());
        }
        if (StringUtil.isNotEmpty(request.getSortKey())) {
            PageHelper.orderBy(request.getSortKey() + " " + request.getSortType());
        }
        PageInfo<TaskSql> pageInfo = new PageInfo<>(taskSqlMapper.find(request, token));

        response.setResult(pageInfo.getList());
        response.setTotalCount(pageInfo.getTotal());

        return response;
    }

    /**
     * 获得对象
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    @Transactional(readOnly = true)
    public TaskSqlGetResponse get(TaskSqlGetRequest request, Token token) {
        TaskSqlGetResponse response = new TaskSqlGetResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        TaskSql po = taskSqlMapper.get(request, token);

        if (po != null) {
            response.setTaskSql(po);
        } else {
            response.addError(ErrorType.BUSINESS_ERROR, Message.GET_FAILURE);
        }

        return response;
    }

    /**
     * 添加或替换Sql任务对象
     *
     * @param taskSql Sql任务对象
     * @return
     */
    public boolean createOrRepeat(TaskSql taskSql) {
        try {
            if ("Cron".equals(taskSql.getTaskType())) {
                scheduler.createOrRepeat(new RunCronTask() {

                    @Override
                    public String cron() {
                        return taskSql.getTypeValue();
                    }

                    @Override
                    public String taskName() {
                        return taskSql.getTaskName();
                    }

                    @Override
                    public String taskId() {
                        return String.valueOf(taskSql.getId());
                    }

                    @Override
                    public void run() {
                        try {
                            SqlUtil.exec(taskSql.getTaskSql());
                        } catch (Exception e) {
                            throw new RuntimeException(String.format("[%s]执行异常！任务ID(%s)", taskName(), taskId()));
                        }
                    }
                });
            }
            if ("DelayRepeat".equals(taskSql.getTaskType())) {
                scheduler.createOrRepeat(new RunDelayRepeatTask() {
                    @Override
                    public Duration interval() {
                        String typeValue = taskSql.getTypeValue();
                        return Duration.ofSeconds(Integer.parseInt(typeValue));
                    }

                    @Override
                    public String taskName() {
                        return taskSql.getTaskName();
                    }

                    @Override
                    public String taskId() {
                        return String.valueOf(taskSql.getId());
                    }

                    @Override
                    public void run() {
                        try {
                            SqlUtil.exec(taskSql.getTaskSql());
                        } catch (Exception e) {
                            throw new RuntimeException(String.format("[%s]执行异常！任务ID(%s)", taskName(), taskId()));
                        }
                    }
                });
            }
            if ("FixRepeat".equals(taskSql.getTaskType())) {
                scheduler.createOrRepeat(new RunFixRepeatTask() {
                    @Override
                    public Duration interval() {
                        String typeValue = taskSql.getTypeValue();
                        return Duration.ofSeconds(Integer.parseInt(typeValue));
                    }

                    @Override
                    public String taskName() {
                        return taskSql.getTaskName();
                    }

                    @Override
                    public String taskId() {
                        return String.valueOf(taskSql.getId());
                    }

                    @Override
                    public void run() {
                        try {
                            SqlUtil.exec(taskSql.getTaskSql());
                        } catch (Exception e) {
                            throw new RuntimeException(String.format("[%s]执行异常！任务ID(%s)", taskName(), taskId()));
                        }
                    }
                });
            }
        } catch (Exception e) {
            return false;
        }
        return true;
    }
}
