package ${basePackage}.system;

import ${basePackage}.frame.auth.Token;
import ${basePackage}.module.system.mgr.TaskSqlManager;
import ${basePackage}.module.system.req.TaskSqlCreateRequest;
import ${basePackage}.module.system.req.TaskSqlDeleteRequest;
import ${basePackage}.module.system.req.TaskSqlFindRequest;
import ${basePackage}.module.system.req.TaskSqlGetRequest;
import ${basePackage}.module.system.req.TaskSqlUpdateRequest;
import ${basePackage}.module.system.rsp.TaskSqlCreateResponse;
import ${basePackage}.module.system.rsp.TaskSqlDeleteResponse;
import ${basePackage}.module.system.rsp.TaskSqlFindResponse;
import ${basePackage}.module.system.rsp.TaskSqlGetResponse;
import ${basePackage}.module.system.rsp.TaskSqlUpdateResponse;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import static junit.framework.TestCase.assertTrue;

/**
 * TaskSqlTest -  - SQL任务测试用例
 *
 * @author author
 * @version 0.0.1
 * @since 2020-05-24
 */
@RunWith(SpringRunner.class)
@SpringBootTest
@Transactional
public class TaskSqlTest {

    @Autowired
    private Token token;

    @Autowired
    private TaskSqlManager taskSqlManager;

    @Test
    public void testCreate() {
        TaskSqlCreateRequest request = new TaskSqlCreateRequest();
        request.setTaskName("任务名称");
        request.setTaskNote("详细注释");
        request.setTaskType("code");
        request.setTypeValue("任务类型值");
        request.setTaskSql("任务SQL");
        request.setValid(true);

        TaskSqlCreateResponse response = taskSqlManager.create(request,token);

        assertTrue(!response.hasError());
    }

    @Test
    public void testDelete() {

        //创建数据
        TaskSqlCreateRequest createRequest = new TaskSqlCreateRequest();
        createRequest.setTaskName("任务名称");
        createRequest.setTaskNote("详细注释");
        createRequest.setTaskType("code");
        createRequest.setTypeValue("任务类型值");
        createRequest.setTaskSql("任务SQL");
        createRequest.setValid(true);

        TaskSqlCreateResponse createResponse = taskSqlManager.create(createRequest,token);

        assertTrue(!createResponse.hasError() && createResponse.getId() > 0);
        //删除数据
        TaskSqlDeleteRequest request = new TaskSqlDeleteRequest();
        request.setId(createResponse.getId());

        TaskSqlDeleteResponse response = taskSqlManager.delete(request,token);

        assertTrue(!response.hasError() && response.getResult() == 1L);
    }

    @Test
    public void testUpdate() {
        //创建数据
        TaskSqlCreateRequest createRequest = new TaskSqlCreateRequest();
        createRequest.setTaskName("任务名称");
        createRequest.setTaskNote("详细注释");
        createRequest.setTaskType("code");
        createRequest.setTypeValue("任务类型值");
        createRequest.setTaskSql("任务SQL");
        createRequest.setValid(true);

        TaskSqlCreateResponse createResponse = taskSqlManager.create(createRequest, token);

        assertTrue(!createResponse.hasError());

        //更新数据
        TaskSqlUpdateRequest request = new TaskSqlUpdateRequest();
        request.setId(createResponse.getId());
        request.setTaskName("任务名称");
        request.setTaskNote("详细注释");
        request.setTaskType("code");
        request.setTypeValue("任务类型值");
        request.setTaskSql("任务SQL");
        request.setValid(true);

        TaskSqlUpdateResponse response = taskSqlManager.update(request,token);

        assertTrue(!response.hasError() && response.getResult() == 1L);
    }

    @Test
    public void testFind() {
        TaskSqlFindRequest request = new TaskSqlFindRequest();
        request.setTaskName("任务名称");
        request.setTaskNote("详细注释");
        request.setTaskType("code");
        request.setValid(true);

        TaskSqlFindResponse response = taskSqlManager.find(request,token);

        assertTrue(!response.hasError());
    }

    @Test
    public void testGet() {
        //创建数据
        TaskSqlCreateRequest createRequest = new TaskSqlCreateRequest();
        createRequest.setTaskName("任务名称");
        createRequest.setTaskNote("详细注释");
        createRequest.setTaskType("code");
        createRequest.setTypeValue("任务类型值");
        createRequest.setTaskSql("任务SQL");
        createRequest.setValid(true);

        TaskSqlCreateResponse createResponse = taskSqlManager.create(createRequest, token);

        assertTrue(!createResponse.hasError());

        //获得数据
        TaskSqlGetRequest request = new TaskSqlGetRequest();
        request.setId(createResponse.getId());

        TaskSqlGetResponse response = taskSqlManager.get(request,token);

        assertTrue(!response.hasError() && response.getTaskSql() != null);
    }
}
